import * as domloaded from "dom-loaded";

import "../css/cloudstudioify.css";
import { ConfigProvider } from "./config";
import { Injector } from "./injectors/injector";
import { InjectorProvider } from "./injectors/injector-provider";
import { renderCloudStudioUrl } from "./utils";

let initChain: Promise<void> = Promise.resolve();
let isInstalling: boolean = false;
let observer: MutationObserver | undefined;

const init = async (injectedByUserClick: boolean = false) => {
  const configProvider = await ConfigProvider.create();
  const config = configProvider.getConfig();

  const cloudstudioExtensionCheckElement = document.getElementById(
    "ExtensionCheck_GitpodBrowserExtension"
  );
  if (cloudstudioExtensionCheckElement) {
    cloudstudioExtensionCheckElement.innerHTML = "installed";
    if (injectedByUserClick) {
      window.open(config.cloudStudioUrl);
    }
    return;
  }

  // Get the first injector that canHandleCurrentPage()
  const injectorProvider = new InjectorProvider(configProvider);
  const injector = injectorProvider.findInjectorForCurrentPage();
  if (!injector) {
    if (injectedByUserClick) {
      window.open(config.cloudStudioUrl);
    }
    return;
  }

  if (injectedByUserClick) {
    window.open(renderCloudStudioUrl(config.cloudStudioUrl));
  }

  // Perform the actual, initial injection
  await injector.inject();
  await domloaded;

  // Observe and update on DOM changes
  updateOnDOMChanges(injector);

  // Listen for config changes
  configProvider.on(() => {
    reinit();
    configProvider.dispose();
  });
};

const injectedByUserClick = (): boolean => {
  const div = document.getElementById("cloudstudio-extension-icon-clicked");
  if (div) {
    div.remove();
    return true;
  }
  return false;
};

const updateOnDOMChanges = (injector: Injector) => {
  observer = new MutationObserver(() => {
    if (!injector.checkIsInjected() && !isInstalling) {
      isInstalling = true;
      injector.update().then(() => (isInstalling = false));
    }
  });
  observer.observe(document, {
    childList: true,
    subtree: true,
    attributes: true,
    characterData: true,
  });
};

/**
 * This method is called on config changes. It resets the state to inital and re-runs init
 */
const reinit = () => {
  initChain
    .then(() => {
      isInstalling = false;
      if (observer) {
        observer.disconnect();
        observer = undefined;
      }
    })
    .then(() => init());
};

initChain = init(injectedByUserClick());
