import * as select from "select-dom";
import { ConfigProvider } from "../config";
import { ButtonInjector, InjectorBase, checkIsBtnUpToDate } from "./injector";
import { renderCloudStudioUrl, makeOpenInPopup } from "../utils";

enum ClassNames {
  BTN_ID = "cloudstudio-btn-nav",
  NAV_BTN_ID = "cloudstudio-btn-nav",
  NAV_BTN_CLASS = "cloudstudio-nav-btn",
  NAV_BTN_CLASS_SELECTOR = ".cloudstudio-nav-btn",
  EDIT_BTN_ID = "cloudstudio-btn-file",
  CSS_REF_BTN_CONTAINER = "cloudstudio-btn-container",
  CSS_REF_NO_CONTAINER = "no-container",
}

export class CodingInjector extends InjectorBase {
  constructor(configProvider: ConfigProvider) {
    console.log("init coding injector");
    super(configProvider, [
      new RepositoryInjector(),
      new MergeInjector(),
      new PublicRepositoryInjector(),
    ]);
  }

  canHandleCurrentPage(): boolean {
    const isCodingRepo = window.location.hostname.endsWith("coding.net");
    console.log("canHandleCurrentPage: ", !!isCodingRepo); // @test
    return !!isCodingRepo;
  }

  checkIsInjected(): boolean {
    console.log("checkIsInjected"); // @TEST
    const button = document.getElementById(`${ClassNames.NAV_BTN_ID}`);
    const currentUrl = renderCloudStudioUrl(this.config.cloudStudioUrl);
    return checkIsBtnUpToDate(button, currentUrl);
  }

  async inject(): Promise<void> {
    console.log("inject"); // @TEST
    if (!this.checkIsInjected()) {
      this.injectButtons();
    }
  }

  async update(): Promise<void> {
    console.log("update"); // @TEST
    this.injectButtons();
  }
}

class RepositoryInjector implements ButtonInjector {
  static readonly PARENT_SELECTOR = "#tree-right-panel";

  isApplicableToCurrentPage(): boolean {
    console.log("isApplicableToCurrentPage");
    const result =
      !!select.exists(RepositoryInjector.PARENT_SELECTOR) &&
      !!select.exists("#clone-panel-id");
    return result;
  }

  inject(currentUrl: string, openAsPopup: boolean) {
    console.log("RepositoryInjector.inject"); // @TEST
    const parent = select(RepositoryInjector.PARENT_SELECTOR);
    if (!parent || !parent.firstElementChild) {
      return;
    }

    const btn = this.renderButton(currentUrl, openAsPopup);
    parent.firstElementChild.appendChild(btn);
  }

  protected renderButton(url: string, openAsPopup: boolean): HTMLElement {
    console.log("renderButton", url);
    const container = document.createElement("div");
    container.className =
      "project-clone-holder d-none d-md-inline-block coding-container";
    container.setAttribute(
      "style",
      `position: absolute;
        right: 175px;
        top: 48px;`
    );

    const container2ndLevel = document.createElement("div");
    container2ndLevel.className =
      "coding-button git-clone-holder js-git-clone-holder";

    const a = document.createElement("a");
    a.id = ClassNames.BTN_ID;
    a.title = "Cloud Studio";
    a.text = "Cloud Studio";
    a.href = url;
    a.target = "_blank";

    if (openAsPopup) {
      makeOpenInPopup(a);
    }

    container2ndLevel.appendChild(a);
    container.appendChild(container2ndLevel);
    return container;
  }
}

class PublicRepositoryInjector implements ButtonInjector {
  static readonly PARENT_SELECTOR = "#root";

  isApplicableToCurrentPage(): boolean {
    const result =
      window.location.pathname.startsWith("/public/") &&
      !!select.exists(PublicRepositoryInjector.PARENT_SELECTOR) &&
      !!select.exists("#clone-panel-id");
    console.log("PublicRepositoryInjector.isApplicableToCurrentPage: ", result);
    return result;
  }

  inject(currentUrl: string, openAsPopup: boolean) {
    console.log("PublicRepositoryInjector.inject"); // @TEST
    const parent = select(PublicRepositoryInjector.PARENT_SELECTOR);
    if (!parent || !parent.firstElementChild) {
      return;
    }

    const btn = this.renderButton(currentUrl, openAsPopup);
    const header = parent.querySelector("header");
    if (header) {
      console.log("header: ", header);
      header.appendChild(btn);
    }
  }

  protected renderButton(url: string, openAsPopup: boolean): HTMLElement {
    console.log("renderButton", url);
    const container = document.createElement("div");
    container.className = "coding-container";
    container.setAttribute(
      "style",
      `position: absolute;
        width: 100%;
        top: 74px;
        pointer-events: none;`
    );

    const container2ndLevel = document.createElement("div");
    container2ndLevel.className = "coding-button";
    container2ndLevel.setAttribute(
      "style",
      `margin: 0 auto;
        transform: translateX(335px);`
    );

    const a = document.createElement("a");
    a.id = ClassNames.BTN_ID;
    a.title = "Cloud Studio";
    a.text = "Cloud Studio";
    a.href = url;
    a.target = "_blank";

    if (openAsPopup) {
      makeOpenInPopup(a);
    }

    container2ndLevel.appendChild(a);
    container.appendChild(container2ndLevel);
    return container;
  }
}

class MergeInjector implements ButtonInjector {
  static readonly PARENT_SELECTOR = "#__MERGE_CONTENT_CONTAINER__";

  isApplicableToCurrentPage(): boolean {
    console.log("MergeInjector.isApplicableToCurrentPage");
    const result =
      !!select.exists(MergeInjector.PARENT_SELECTOR) &&
      window.location.pathname.includes("/merge/");
    return result;
  }

  inject(currentUrl: string, openAsPopup: boolean) {
    console.log("MergeInjector.inject"); // @TEST
    const parent = select(MergeInjector.PARENT_SELECTOR);
    if (!parent || !parent.firstElementChild) {
      return;
    }

    const btn = this.renderButton(currentUrl, openAsPopup);
    parent.appendChild(btn);
  }

  protected renderButton(url: string, openAsPopup: boolean): HTMLElement {
    console.log("MergeInjector.renderButton", url);
    const container = document.createElement("div");
    container.className = "coding-container";
    container.setAttribute(
      "style",
      `position: absolute;
        top: 42px;
        right: 15%;
        z-index: 100;`
    );

    const container2ndLevel = document.createElement("div");
    container2ndLevel.className = "coding-button";

    const a = document.createElement("a");
    a.id = ClassNames.BTN_ID;
    a.title = "Cloud Studio";
    a.text = "Cloud Studio";
    a.href = url;
    a.target = "_blank";

    if (openAsPopup) {
      makeOpenInPopup(a);
    }

    container2ndLevel.appendChild(a);
    container.appendChild(container2ndLevel);
    return container;
  }
}
